@@ -31,7 +31,9 @@
 #ifdef HAVE_REGEXEC
 #include <regex.h>
 #endif
-#ifdef HAVE_LIBIDN
+#ifdef HAVE_LIBIDN2
+#include <idn2.h>
+#elif defined HAVE_LIBIDN
 #include <idna.h>
 #endif
 #ifdef HAVE_INET_PTON
@@ -126,7 +128,8 @@ int main(int argc, char *argv[])
     int longindex;
 #endif
 
-    int ch, nopar = 0, fstringlen = 64;
+    int ch, nopar = 0;
+    size_t fstringlen = 64;
     const char *server = NULL, *port = NULL;
     char *qstring, *fstring;
     int ret;
@@ -653,7 +656,7 @@ char *queryformat(const char *server, const char *flags, const char *query)
 	simple_recode_input_charset = "utf-8";	/* then try UTF-8 */
 #endif
 
-#ifdef HAVE_LIBIDN
+#if defined HAVE_LIBIDN || defined HAVE_LIBIDN2
 # define DENIC_PARAM_ACE ",ace"
 #else
 # define DENIC_PARAM_ACE ""
@@ -939,9 +942,7 @@ int openconn(const char *server, const char *port)
     hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_flags = AI_ADDRCONFIG;
-#ifdef HAVE_LIBIDN
     hints.ai_flags |= AI_IDN;
-#endif
 
     if ((err = getaddrinfo(server, port ? port : "nicname", &hints, &res))
 	    != 0) {
@@ -1050,13 +1051,13 @@ int connect_with_timeout(int fd, const struct sockaddr *addr,
     return 0;
 }
 
-void alarm_handler(int signum)
+void NORETURN alarm_handler(int signum)
 {
     close(sockfd);
     err_quit(_("Timeout."));
 }
 
-void sighandler(int signum)
+void NORETURN sighandler(int signum)
 {
     close(sockfd);
     err_quit(_("Interrupted by signal %d..."), signum);
@@ -1133,7 +1134,7 @@ const char *is_new_gtld(const char *s)
 	if (in_domain(s, new_gtlds[i]))
 	    return new_gtlds[i];
 
-    return 0;
+    return NULL;
 }
 
 /*
@@ -1145,7 +1146,7 @@ const char *is_new_gtld(const char *s)
 char *normalize_domain(const char *dom)
 {
     char *p, *ret;
-#ifdef HAVE_LIBIDN
+#if defined HAVE_LIBIDN || defined HAVE_LIBIDN2
     char *domain_start = NULL;
 #endif
 
@@ -1160,7 +1161,7 @@ char *normalize_domain(const char *dom)
 	p--;
     }
 
-#ifdef HAVE_LIBIDN
+#if defined HAVE_LIBIDN || defined HAVE_LIBIDN2
     /* find the start of the last word if there are spaces in the query */
     for (p = ret; *p; p++)
 	if (*p == ' ')
@@ -1170,8 +1171,13 @@ char *normalize_domain(const char *dom)
 	char *q, *r;
 	int prefix_len;
 
+#ifdef HAVE_LIBIDN2
+	if (idn2_lookup_ul(domain_start, &q, IDN2_NONTRANSITIONAL) != IDN2_OK)
+	    return ret;
+#else
 	if (idna_to_ascii_lz(domain_start, &q, 0) != IDNA_SUCCESS)
 	    return ret;
+#endif
 
 	/* reassemble the original query in a new buffer */
 	prefix_len = domain_start - ret;
@@ -1186,8 +1192,13 @@ char *normalize_domain(const char *dom)
     } else {
 	char *q;
 
+#ifdef HAVE_LIBIDN2
+	if (idn2_lookup_ul(ret, &q, IDN2_NONTRANSITIONAL) != IDN2_OK)
+	    return ret;
+#else
 	if (idna_to_ascii_lz(ret, &q, 0) != IDNA_SUCCESS)
 	    return ret;
+#endif
 
 	free(ret);
 	return q;
@@ -1232,7 +1243,7 @@ void split_server_port(const char *const input,
     }
 
     /* change the server name to lower case */
-    for (p = (char *) *server; *p && *p != '\0'; p++)
+    for (p = (char *) *server; *p; p++)
 	*p = tolower(*p);
 }
 
@@ -1268,7 +1279,7 @@ char *convert_6to4(const char *s)
     }
 
     new = malloc(sizeof("255.255.255.255"));
-    sprintf(new, "%d.%d.%d.%d", a >> 8, a & 0xff, b >> 8, b & 0xff);
+    sprintf(new, "%ud.%ud.%ud.%ud", a >> 8, a & 0xff, b >> 8, b & 0xff);
 #endif
 
     return new;
@@ -1298,7 +1309,7 @@ char *convert_teredo(const char *s)
     a ^= 0xffff;
     b ^= 0xffff;
     new = malloc(sizeof("255.255.255.255"));
-    sprintf(new, "%d.%d.%d.%d", a >> 8, a & 0xff, b >> 8, b & 0xff);
+    sprintf(new, "%ud.%ud.%ud.%ud", a >> 8, a & 0xff, b >> 8, b & 0xff);
 #endif
 
     return new;
@@ -1378,7 +1389,7 @@ int isasciidigit(const char c) {
 
 /* http://www.ripe.net/ripe/docs/databaseref-manual.html */
 
-void usage(int error)
+void NORETURN usage(int error)
 {
     fprintf((EXIT_SUCCESS == error) ? stdout : stderr, _(
 "Usage: whois [OPTION]... OBJECT...\n\n"
